<?php

App::uses('BuzzVouchersAppController', 'BuzzVouchers.Controller');
App::uses('CustomerAddress', 'BuzzCustomers.Model');

class GiftVoucherPurchasesController extends BuzzVouchersAppController {

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'index',
			'vouchers',
			'basket',
			'remove',
			'delivery',
			'checkout',
			'confirmation'
		));
	}

/**
 * Gift vouchers landing page - will mostly get bypassed by homepage/activity pages
 *
 * @return void
 */
	public function index() {
		if (!empty($this->request->data)) {

			if (!empty($this->request->data['GiftVoucherPurchase']['voucher_activity_id'])) {
				// Forward to the correct vouchers step.
				return $this->redirect([
					'action' => 'vouchers',
					$this->request->data['GiftVoucherPurchase']['voucher_activity_id']
				]);
			} else {
				$this->Session->setFlash(
					__d('BuzzVouchers', 'Please select an activity'),
					'flash_fail'
				);
			}

		}

		$this->loadModel('BuzzVouchers.VoucherActivity');
		$voucherActivities = $this->VoucherActivity->getListed();
		$voucherActivities = Hash::combine($voucherActivities, '{n}.VoucherActivity.id', '{n}.VoucherActivity.name');

		if (count($voucherActivities) === 1) {
			return $this->redirect([
				'action' => 'vouchers'
			]);
		} elseif (empty($voucherActivities)) {
			// No activities have been setup or created so throw an exception
			throw new NotFoundException();
		}

		$this->set(compact('voucherActivities'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/index';

		return;
	}

/**
 * Vouchers step - add gift vouchers to the basket
 *
 * @param int $voucherActivityId Voucher activity ID
 * @return void
 */
	public function vouchers($voucherActivityId = null) {
		$GiftVoucherPurchase = $this->{$this->modelClass};
		$this->loadModel('BuzzVouchers.VoucherActivity');

		if (!empty($this->request->data)) {

			$result = $GiftVoucherPurchase->addItems($this->request->data['GiftVoucherPurchaseItem']);
			if ($result === true) {
				return $this->redirect(['action' => 'basket']);
			} else {
				$this->Session->setFlash(
					__d('BuzzVouchers', 'Please select a voucher'),
					'flash_fail'
				);
			}

		}

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		if (empty($voucherActivityId)) {
			// Check if the site is configured to only have one listed
			// voucher activity. This is needed to prevent a redirect
			// loop with this and the previous step.
			$voucherActivityId = $this->VoucherActivity->onlyAvailableActivity();
		}

		if ($voucherActivityId === false) {
			throw new NotFoundException();
		}

		// Get the vouchers and activity details for generating the form.
		$data = $this->VoucherActivity->getPurchasableVoucherActivity($voucherActivityId);

		if (empty($data)) {
			throw new NotFoundException();
		}

		if (empty($this->request->data['GiftVoucherPurchaseItem']) && !empty($basket['GiftVoucherPurchaseItem'])) {
			// Populate the form with existing basket items.
			$this->request->data['GiftVoucherPurchaseItem'] = Hash::combine(
				$basket['GiftVoucherPurchaseItem'],
				'{n}.gift_voucher_id',
				'{n}'
			);
		}

		$previousStep = ['action' => 'index'];

		$this->set(compact('data', 'basket', 'previousStep'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/vouchers';

		return;
	}

/**
 * Basket step - view the basket
 *
 * @return void
 */
	public function basket() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		if (!empty($this->request->data['GiftVoucherPurchase']['voucher_activity_id'])) {
			// Redirect to the activity so that the user can add more vouchers.
			return $this->redirect([
				'action' => 'vouchers',
				$this->request->data['GiftVoucherPurchase']['voucher_activity_id']
			]);
		} elseif (!empty($this->request->data)) {
			// Update the basket.
			$GiftVoucherPurchase->updateItems($this->request->data['GiftVoucherPurchaseItem']);
		}

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		// Populate the form.
		if (empty($this->request->data)) {
			$this->request->data = $basket;
		}

		$previousStep = ['action' => 'index'];
		if (!empty($basket['GiftVoucherPurchaseItem'])) {
			$items = Hash::sort($basket['GiftVoucherPurchaseItem'], '{n}.modified', 'desc');
			$previousStep = array(
				'action' => 'vouchers',
				$items[0]['GiftVoucher']['voucher_activity_id']
			);
		}
		$nextStep = ['action' => 'delivery'];

		// Get the voucher activities to populate the form.
		$voucherActivities = ClassRegistry::init('BuzzVouchers.VoucherActivity')->translatedList();

		$this->set(compact('basket', 'previousStep', 'nextStep', 'voucherActivities'));
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/basket';
		if (empty($basket['GiftVoucherPurchaseItem'])) {
			$this->view = 'BuzzVouchers./GiftVoucherPurchases/empty_basket';
		}

		return;
	}

/**
 * Remove item from the basket.
 *
 * @param int $id GiftVoucherPurchaseItem record ID
 * @return void
 */
	public function remove($id) {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		if ($GiftVoucherPurchase->removeItem($id) === false) {
			$this->Session->setFlash(
				__d('BuzzVouchers', 'Unable to remove item'),
				'flash_fail'
			);
		}

		return $this->redirect(['action' => 'basket']);
	}

/**
 * Delivery step
 *
 * @return void
 */
	public function delivery() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		if (empty($basket['GiftVoucherPurchaseItem'])) {
			$this->Session->setFlash(
				__d('BuzzVouchers', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect([
				'action' => 'index'
			]);
		}

		if (!empty($this->request->data['GiftVoucherPurchase']['gift_voucher_delivery_option_id'])) {

			$deliveryOption = (int)$this->request->data['GiftVoucherPurchase']['gift_voucher_delivery_option_id'];

			$result = $GiftVoucherPurchase->addDeliveryDetails(
				$deliveryOption,
				empty($this->request->data['CustomerAddress'][$deliveryOption]) ? null : $this->request->data['CustomerAddress'][$deliveryOption],
				$this->request->data['GiftVoucherPurchase']['gift_message']
			);
			if ($result === true) {
				return $this->redirect(['action' => 'checkout']);
			} else {
				$this->Session->setFlash(
					__('Please correct the errors below'),
					'flash_fail'
				);
			}

		}

		$deliveryOptions = $GiftVoucherPurchase->GiftVoucherDeliveryOption->getOptions();

		if (empty($this->request->data)) {

			$this->request->data = $basket;

			$deliveryOptionIds = Hash::extract($deliveryOptions, '{n}.GiftVoucherDeliveryOption.id');

			if (empty($basket['GiftVoucherPurchase']['delivery_address_id'])) {
				// Set the address field defaults (these are configured by the
				// CustomerAddress plugin to make it easier to override for each site).
				foreach ($deliveryOptionIds as $deliveryOptionId) {
					$this->request->data['CustomerAddress'][$deliveryOptionId]['country_id'] = CustomerAddress::getDefaultCountry();
					$this->request->data['CustomerAddress'][$deliveryOptionId]['us_state_id'] = CustomerAddress::getDefaultUsState();
				}
			} else {
				// Need to set the correct values for the address fields.
				foreach ($deliveryOptionIds as $deliveryOptionId) {
					$this->request->data['CustomerAddress'][$deliveryOptionId] = $basket['DeliveryAddress'];
				}
			}

		}

		$previousStep = ['action' => 'basket'];

		$this->set(compact('basket', 'deliveryOptions', 'previousStep'));
		$this->_populateLookups();
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/delivery';

		return;
	}

/**
 * Checkout step
 *
 * @return void
 */
	public function checkout() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		if (empty($basket['GiftVoucherPurchaseItem'])) {
			$this->Session->setFlash(
				__d('BuzzVouchers', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect([
				'action' => 'index'
			]);
		} elseif (empty($basket['GiftVoucherPurchase']['gift_voucher_delivery_option_id'])) {
			$this->Session->setFlash(
				__d('BuzzVouchers', 'Please select a delivery option'),
				'flash_fail'
			);
			return $this->redirect([
				'action' => 'delivery'
			]);
		}

		if (!empty($this->request->data)) {

			$result = $GiftVoucherPurchase->addBillingDetails($this->request->data['CustomerAddress']);
			if ($result === true) {
				// @todo handle payment
				$this->_completePurchase();
			} else {
				$this->Session->setFlash(
					__('Please correct the errors below'),
					'flash_fail'
				);
			}

		} else {

			// Set the address field defaults (these are configured by the
			// CustomerAddress plugin to make it easier to override for each site).
			$this->request->data['CustomerAddress']['country_id'] = CustomerAddress::getDefaultCountry();
			$this->request->data['CustomerAddress']['us_state_id'] = CustomerAddress::getDefaultUsState();

		}

		// Get the basket.
		$basket = $GiftVoucherPurchase->getBasket();

		$previousStep = ['action' => 'delivery'];

		$this->set(compact('basket', 'previousStep'));
		$this->_populateLookups();
		$this->view = 'BuzzVouchers./GiftVoucherPurchases/checkout';

		return;
	}

/**
 * Confirmation page
 *
 * @param int $id Basket ID
 * @return void
 */
	public function confirmation($id) {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		if ($GiftVoucherPurchase->hashBasketId($id) === $this->Session->read('GiftVoucherPurchase.hash')) {

			$this->Session->delete('GiftVoucherPurchase.hash');

			$basket = $GiftVoucherPurchase->getPurchase($id);

			$this->set(compact('basket'));
			$this->view = 'BuzzVouchers./GiftVoucherPurchases/confirmation';

		} else {

			throw new ForbiddenException();

		}

		return;
	}

/**
 * Complete the purchase (after payment received)
 *
 * @return void Redirects to confirmation page
 */
	protected function _completePurchase() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		$basketId = $GiftVoucherPurchase->getBasketId();
		$this->Session->write('GiftVoucherPurchase.hash', $GiftVoucherPurchase->hashBasketId());

		$GiftVoucherPurchase->completePurchase(true);

		return $this->redirect([
			'action' => 'confirmation',
			$basketId
		]);
	}

/**
 * Populate lookups for address fields.
 *
 * @return void
 */
	protected function _populateLookups() {
		$GiftVoucherPurchase = $this->{$this->modelClass};

		$countries = ClassRegistry::init('BuzzCustomers.Country')->translatedList();
		$usStates = $GiftVoucherPurchase->CustomerAddress->UsState->find('list');

		$this->set(compact('countries', 'usStates'));

		return;
	}

}
